"use strict";
exports.__esModule = true;
var Primus = require("Primus");
var Q = require("q");
var USMARTClient = (function () {
    function USMARTClient(serviceName, auth, debug) {
        if (debug === void 0) { debug = false; }
        var _this = this;
        this.messageCount = 1;
        this.messagePromise = {};
        this.authInfo = auth;
        this.socket = Primus.createSocket({
            transformer: "engine.io"
        });
        this.client = new this.socket(this.getURL(serviceName, debug), {
            transport: {
                extraHeaders: {
                    "api-key-id": auth.keyId,
                    "api-key-secret": auth.keySecret
                }
            }
        });
        this.client.on("data", function (data) {
            if (typeof data.messageCount !== "undefined" && _this.messagePromise.hasOwnProperty(data.messageCount)) {
                var listenToProgres = _this.messagePromise[data.messageCount].listenToProgress;
                var isReferenceSet = !!_this.messagePromise[data.messageCount].reference;
                var doesReferenceMatch = isReferenceSet
                    ? _this.messagePromise[data.messageCount].reference === data.status.reference
                    : false;
                if (listenToProgres && !isReferenceSet) {
                    _this.messagePromise[data.messageCount].reference = data.reference;
                    var currentMessageCount = _this.messageCount++;
                    _this.messagePromise[currentMessageCount] = _this.messagePromise[currentMessageCount - 1];
                }
                else if (listenToProgres && doesReferenceMatch) {
                    _this.messagePromise[data.messageCount].promise.notify(data.status);
                }
                else if (!listenToProgres) {
                    _this.messagePromise[data.messageCount].promise.resolve(data);
                }
            }
        });
    }
    USMARTClient.prototype.actionWithParams = function (action, params, listenToProgress) {
        if (listenToProgress === void 0) { listenToProgress = false; }
        var currentMessageCount = this.messageCount++;
        var deferred = Q.defer();
        params.action = action;
        this.client.write({
            event: "action",
            params: params
        });
        this.messagePromise[currentMessageCount] = {
            action: action,
            listenToProgress: listenToProgress,
            promise: deferred
        };
        return deferred.promise;
    };
    USMARTClient.prototype.actionWithParamsAndListener = function (action, params) {
        return this.actionWithParams(action, params, true);
    };
    USMARTClient.prototype.getURL = function (serviceName, debug) {
        return "not-set";
    };
    return USMARTClient;
}());
exports.USMARTClient = USMARTClient;
